<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco01007UtilizaSubstituiPratosCardapioBase extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $connection;
    protected $retornos = array();

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService   = $crudService;
        $this->environment   = $environment;
        $this->entityManager = $entityManager;
        $this->connection    = $entityManager->getConnection();
    }
    
    public function substituirPratosCardBase(DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        try {
            $this->connection->beginTransaction();
            foreach($rows as $row) {
                $this->updatePratoCardBase($row, $this->connection);
            }
            
            $this->connection->commit();
            $response->addDataSet(new DataSet("cardapiosAlterados", $this->retornos));
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    private function updatePratoCardBase($row, $connection) {
        
        $sql = "UPDATE PRATCOMPCARDBASE
                   SET PRATCOMPCARDBASE.CDFILAUXPRAT    = :CDFILAUXPRATDEST,
                       PRATCOMPCARDBASE.CDPRATO         = :CDPRATODEST,
                       PRATCOMPCARDBASE.DTULTATU        = SYSDATE,
                       PRATCOMPCARDBASE.NRORGULTATU     = :NRORGULTATU,
                       PRATCOMPCARDBASE.CDOPERULTATU    = :CDOPERULTATU
                 WHERE PRATCOMPCARDBASE.CDFILAUXPRAT    = :CDFILAUXPRAT
                   AND PRATCOMPCARDBASE.CDPRATO         = :CDPRATO 
                   AND EXISTS ( SELECT 1
                                  FROM COMPCARDBASE 
                                 WHERE COMPCARDBASE.NRCARDAPIOBASE = :NRCARDAPIOBASE
                                   AND COMPCARDBASE.NRCOMPCARDBASE = PRATCOMPCARDBASE.NRCOMPCARDBASE )
                   AND PRATCOMPCARDBASE.NRORG        = :NRORG";
                   
        $params = array(
            "CDFILAUXPRATDEST" => $row["CDFILAUXPRATDEST"],
            "CDPRATODEST"      => $row["CDPRATODEST"],
            "CDFILAUXPRAT"     => $row["CDFILAUXPRAT"],
            "CDPRATO"          => $row["CDPRATO"],
            "NRCARDAPIOBASE"   => $row["NRCARDAPIOBASE"],
            "CDOPERULTATU"     => $this->environment->getCdOperador(),
            "NRORGULTATU"      => $this->environment->getNrOrgTrab(),
            "NRORG"            => $this->environment->getNrOrgTrab()
        );
        
        $stmt = $connection->prepare($sql);
        $stmt->execute($params);
        
        array_push($this->retornos, array(
            "CDCARDAPIOBASE" => $row["CDCARDAPIOBASE"]
        ));
        
    }

}
