<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\SMTP;
use Teknisa\TCO\Services\Tco01100SMTP;


class Tco01100RelatoriosPropostaComercial extends \Zeedhi\Framework\Controller\Report {
    
    protected $reportService; 
    protected $environment; 
    protected $fileUploader;
    protected $fileServerURL;
    protected $entityManager;
    protected $birtStrategy; 
    protected $listas;
    protected $novoCodigo;
    
    public function __construct($reportService, $environment, $fileUploader,$fileServerURL,$entityManager, $birtStrategy, $listas,$novoCodigo,$connection) {
        $this->reportService = $reportService; 
        $this->environment = $environment; 
        $this->fileUploader = $fileUploader;
        $this->fileServerURL = $fileServerURL;
        $this->entityManager = $entityManager;
        $this->birtStrategy = $birtStrategy; 
        $this->listas = $listas;
        $this->novoCodigo = $novoCodigo;
        $this->connection = $connection;
    }
    
    protected $reportMapping = array ( 
        'TCO01103' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRPROPOSTACOMERCIAL'  => 'NRPROPOSTACOMERCIAL',
                        'NRVERSAOPROPCOM'      => 'NRVERSAOPROPCOM',
                        'NRSERVICOPROPCOM'     => 'NRSERVICOPROPCOM',
                        'NRDETALHESERVPROPCOM' => 'NRDETALHESERVPROPCOM',
                        'NRHISTNOTIFPROPCOM'   => 'NRHISTNOTIFPROPCOM',
                        'NRORG'                => 'NRORG'
                    )
            ),
        'TCO01104' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'NRORG'                => 'P_NRORG',
                        'NRPROPOSTACOMERCIAL'  => 'P_NRPROPOSTACOMERCIAL',
                        'NRVERSAOPROPCOM'      => 'P_SIMULACOES',
                        'NRSERVICOPROPCOM'     => 'P_SERVICOS'
                    )
            )
    );
    
    public function constroiMensagemParaEmail($assunto, $motivo, $nrPropostaComercial){
        $mensagem = "<html style='padding: 0px; margin: 0px;'>
                       <head>
                          <style>table{border: 1px solid #cccaca; width: 550px; background-color: #dbdbdb;}p, pre, td{font-family: Verdana, Geneva, sans-serif !important; color: #000000; font-size: 14px; line-height: 18px;}.description{padding: 20px; padding-bottom: 10px; padding-top: 4px;}.title{padding: 20px; padding-top: 10px; padding-bottom: 0px;}.title div{font-weight: bold; width: 100%; border-bottom: 1px solid #000;}.footer p{color: #00528C; font-weight: 300; margin-bottom: 40px;}.header .logo{padding-bottom: 10px;}.logo img{width: 200px; margin-left: 20px; margin-top: 15px;}</style>
                          <link rel='stylesheet' href='/zimbra/css/msgview.css?v=120105100505'>
                       </head>
                       <body class='MsgBody MsgBody-html' style='margin: 0px; height: auto;'>
                          <center>
                             <table cellpadding='0' cellspacing='0'>
                                <tbody>
                                   <tr class='header'>
                                      <td class='logo'>
                                         <p><img dfsrc='https://www.teknisa.com/imagens/logo-teknisa.png' src='https://www.teknisa.com/imagens/logo-teknisa.png'></p>
                                      </td>
                                   </tr>
                                   <tr>
                                      <td class='title'>
                                         <div> ".$assunto." </div>
                                      </td>
                                   </tr>
                                ";
        $mensagem .= "<tr> <td class='description'> <span> <b>".$motivo.": ".$nrPropostaComercial."</b> </span> </td> </tr>";
        
        $mensagem .= "</tbody> </table> </center> </body> </html>";
        return $mensagem;
    }
    
    public function convertToB64( $filePath ) {
        $file = fopen($filePath, 'r');
        $size = filesize($filePath);
        $content = fread($file, $size);
        return [chunk_split(base64_encode($content)), $size];
    }
    
    public function sendEmail(DTO\Request\Row $request, DTO\Response $response){
        
        try{
            $row = $request->getRow();
            $row["__report_name"] = "TCO01104";
            $row["NRORG"] = $this->environment->getNrorgTrab();
            if(isset($row[1]["NRSERVICOPROPCOM"])){
                if(is_array($row[1]["NRSERVICOPROPCOM"])){
                    $row[1]["NRSERVICOPROPCOM"] = empty($row[1]["NRSERVICOPROPCOM"]) ? "T" : $this->listas->criarListaValores(implode("_", $row[1]["NRSERVICOPROPCOM"]));
                }else{
                    $row[1]["NRSERVICOPROPCOM"] = empty($row[1]["NRSERVICOPROPCOM"]) ? "T" : $row[1]["NRSERVICOPROPCOM"];
                }
            }else{
                $row[1]["NRSERVICOPROPCOM"] = "T";
            }
    
            $parametrosRelatorio = array (
                            'P_NRORG'                => $row[1]["NRORG"],
                            'P_NRPROPOSTACOMERCIAL'  => $row[1]["NRPROPOSTACOMERCIAL"],
                            'P_SIMULACOES'           => $row[1]["NRVERSAOPROPCOM"],
                            'P_SERVICOS'             => $row[1]["NRSERVICOPROPCOM"]
            );
           
            $reportName = parent::getReportName($row);
            $reportStrategy = parent::getReportStrategy($reportName);
            $assuntoEmail = $row[1]["ASSUNTO_EMAIL"];
            $smtp = new Tco01100SMTP($this->connection, $this->environment);
            $smtp->setSubject($assuntoEmail);
            $url = $this->reportService->createRemoteReport($reportStrategy, $reportName, $parametrosRelatorio, true);
            $nomeArquivo = $row[1]["NOME_ARQUIVO"];
            $tituloEmail = $row[1]["TITULO_EMAIL"];
            $corpoEmail = $row[1]["CORPO_EMAIL"];
            $dsmotivo = $row[1]["DSMOTIVO"];
            $reportFormat = isset($row[1]['QR2TYPE']) ? $row[1]['QR2TYPE'] : 'pdf';
            
            $url = str_replace('pdf', $reportFormat, $url);
            
            try{
                $filePath = $smtp->addReportToAttachment($url, $nomeArquivo, strtolower($reportFormat));
            }catch(\Exception $e){
                throw new \Exception("TCO01100_ERRO_ANEXAR_ARQ_EMAIL###".$e->getMessage(), 1, $e);    
            }
            
            try {
                list($fileInB64, $size) = $this->convertToB64( $filePath );
                
                $row['FILE'] = [[
                    'b64File' => 'data:application/pdf;base64,'.$fileInB64,
                    'name' => basename($filePath),
                    'size' => $size,
                    'type' => 'application/pdf'
                ]];
    
                $this->fileUploader->uploadFilesInRow($row, 'FILE', 'Tco/Documentos');
                $file = $row->get('FILE')[0];
            }catch(\Exception $e){
                throw new \Exception("TCO01100_ERRO_COMUNICACAO_SERVIDOR_ARQUIVOS###". $e->getMessage());    
            }
            
            try{
                $mensagem = $this->constroiMensagemParaEmail($tituloEmail, $corpoEmail, $row[1]["NRPROPOSTACOMERCIAL"]);
                $smtp->sendMail($mensagem, $row[0]);
                $smtp->deleteTemporaryFiles();
            }catch(\Exception $e){
                throw new \Exception("TCO01100_ERRO_AO_ENVIAR_EMAIL###". $e->getMessage()); 
            }
            
            try{    
                $parametrosHistorico = array (
                                'NRORG'                => $row[1]["NRORG"],
                                'NRPROPOSTACOMERCIAL'  => $row[1]["NRPROPOSTACOMERCIAL"],
                                'NRVERSAOPROPCOM'      => $row[1]["NRVERSAOPROPCOM"],
                                'NRSERVICOPROPCOM'     => $row[1]["NRSERVICOPROPCOM"],
                                'URL'                  => $url,
                                'FILE'                 => $file,
                                'SERVICO_DETALHE'      => $row[1]["SERVICO_DETALHE"],
                                'DSMOTIVO'             => $dsmotivo
                );
                
                $this->salvaHistorico($parametrosHistorico);
            }catch(\Exception $e){
                throw new \Exception("TCO01100_ERRO_AO_SALVAR_HISTORICO###". $e->getMessage());
            }
            
            $response->addDataSet(new DataSet ('sucess', $parametrosHistorico));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
        
    }
    
    public function insertHistorico($dados){
        
        $sql = "INSERT INTO HISTNOTIFPROPCOM
                   (NRHISTNOTIFPROPCOM, NRSERVICOPROPCOM,
                    IDORIGEM, IDDESTINO, IDMOTIVO,
                    DSOBSERVACAO, NRDETALHESERVPROPCOM,
                    NRORG, NRORGINCLUSAO, NRORGULTATU,
                    CDOPERINCLUSAO, CDOPERULTATU,
                    DTINCLUSAO, DTULTATU,
                    DSARQUIVOENVIADO)
                VALUES
                   (:P_NRHISTNOTIFPROPCOM, :P_NRSERVICOPROPCOM,
                    'PLANEJAMENTO', 'GN', 'ENVIOEMAIL',
                    :P_DSMOTIVO, :P_NRDETALHESERVPROPCOM,
                    :P_NRORG, :P_NRORG, :P_NRORG,
                    :P_CDOPERADOR, :P_CDOPERADOR,
                    SYSDATE, SYSDATE,
                    :P_DSARQUIVOENVIADO)";
			       
		
		$query = $this->entityManager->getConnection()->prepare($sql);
		$query->execute($dados);
				
	}
    
    public function salvaHistorico($parametrosHistorico){
        
        try{
            foreach( $parametrosHistorico["SERVICO_DETALHE"] as $data) {
               
               $nrhistnotifpropcom = $this->novoCodigo->getCodeByTableName('HISTNOTIFPROPCOM', $this->environment->getNrOrg());
               $cdoperador = $this->environment->getCdOperador();
               
               $dados = array(
                    "P_NRHISTNOTIFPROPCOM" => $nrhistnotifpropcom,
        			"P_NRSERVICOPROPCOM" => $data[0],	
        			"P_NRDETALHESERVPROPCOM" => $data[1],
        			"P_NRORG" => $parametrosHistorico['NRORG'],
        			"P_CDOPERADOR" => $cdoperador,
        			"P_DSARQUIVOENVIADO" => $parametrosHistorico['FILE']['path'],
        			"P_DSMOTIVO" => $parametrosHistorico['DSMOTIVO']
        		); 

    	        $this->connection->beginTransaction();
    	        
			    $this->insertHistorico($dados);
			
			    $this->connection->commit();
    				
    			}
        }catch (\Exception $e) {
            	$this->connection->rollBack(); 
            	$response->setError(new Error($e->getMessage(), 500));
        } 

    }
    
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row["__report_name"] = $row["REPORT_NAME"];
        $row["NRORG"] = $this->environment->getNrorgTrab();
        
        if (isset($row['QR2TYPE']) && $row['QR2TYPE']){
          $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        }
        
        if(isset($row["NRVERSAOPROPCOM"])){
            if(is_array($row["NRVERSAOPROPCOM"])){
                $row["NRVERSAOPROPCOM"] = empty($row["NRVERSAOPROPCOM"]) ? "T" : $this->listas->criarListaValores(implode("_", $row["NRVERSAOPROPCOM"]));
            }else{
                $row["NRVERSAOPROPCOM"] = empty($row["NRVERSAOPROPCOM"]) ? "T" : $row["NRVERSAOPROPCOM"];
            }
        }else{
            $row["NRVERSAOPROPCOM"] = "T";
        }
      
        if(isset($row["NRSERVICOPROPCOM"])){
            if(is_array($row["NRSERVICOPROPCOM"])){
                $row["NRSERVICOPROPCOM"] = empty($row["NRSERVICOPROPCOM"]) ? "T" : $this->listas->criarListaValores(implode("_", $row["NRSERVICOPROPCOM"]));
            }else{
                $row["NRSERVICOPROPCOM"] = empty($row["NRSERVICOPROPCOM"]) ? "T" : $row["NRSERVICOPROPCOM"];
            }
        }else{
            $row["NRSERVICOPROPCOM"] = "T";
        }
        
    }
}